from pathlib import Path
import tempfile
import shutil

import pytest

from bh_util.flagfolder import FlagFolder

@pytest.fixture(scope='module')
def sandbox():
    box = tempfile.mkdtemp()
    yield box
    assert Path(box).exists()
    shutil.rmtree(box)
    assert not Path(box).exists()

@pytest.fixture()
def tmp(sandbox):
    yield Path(tempfile.mkdtemp(dir=str(sandbox)))

@pytest.fixture()
def ff(tmp):
    return FlagFolder(tmp)

###########################

def test_flags__after_init(ff):
    assert ff.flags() == []

def test_flags__after_set(ff):
    ff.set('foo')
    assert ff.flags() == ['foo']

def test_flags__after_set2(ff):
    ff.set('foo')
    ff.set('bar')
    assert ff.flags() == ['bar', 'foo']

def test_clear(ff):
    ff.set('foo')
    ff.set('bar')
    ff.clear('foo')
    assert ff.flags() == ['bar']

def test_clear_all(ff):
    ff.set('foo')
    ff.set('bar')
    ff.clear_all()
    assert ff.flags() == []


def test_stat__after_init(ff):
    assert ff.stat('foo') == False
def test_stat__after_set(ff):
    ff.set('foo')
    assert ff.stat('foo') == True
def test_stat__after_clear(ff):
    ff.set('foo')
    ff.clear('foo')
    assert ff.stat('foo') == False

def test_dunder_init__can_create_deep_folder(tmp):
    path = tmp/'a/b/c'
    assert not path.exists()
    FlagFolder(path)
    assert path.exists()

def test_dunder_init__flags_are_retained(tmp):
    old =  FlagFolder(tmp)
    old.set('foo')
    new = FlagFolder(tmp)
    assert new.stat('foo')





#!/usr/bin/env python3
import sys
from pathlib import Path

import pytest

from bh_util.debug import Debug
ATTRIBNAME='_bh_debug'
@pytest.fixture()
def pill():
    try:
        del sys._bh_debug
    except AttributeError:
        pass
    pill = KillPill()
    yield KillPill()
def test_dunder_init__True_False():
    debug = Debug(True)
    assert debug
    debug = Debug(False)
    assert not debug

def test_dunder_init__True_null():
    debug = Debug(True)
    assert debug
    debug = Debug()
    assert not debug

def test_turn_on():
    debug = Debug()
    assert not debug
    debug.turn_on()
    assert debug

def test_turn_off():
    debug = Debug(True)
    assert debug
    debug.turn_off()
    assert not debug

def test_err__debug_on(capsys):
    debug = Debug(True)
    debug.err("Hello")
    captured = capsys.readouterr()
    assert captured.err == "Hello\n"
    assert captured.out == ""

def test_err__debug_on_test_end(capsys):
    debug = Debug(True)
    debug.err("Hello", end='')
    captured = capsys.readouterr()
    assert captured.err == "Hello"
    assert captured.out == ""

def test_err__debug_off(capsys):
    debug = Debug(False)
    debug.err("Hello", end='')
    captured = capsys.readouterr()
    assert captured.err == ""
    assert captured.out == ""





#!/usr/bin/env python3

from pathlib import Path

import pytest

from bh_util.killpill import KillPill

@pytest.fixture()
def pill():
    pill = KillPill()
    yield KillPill()

def test_dunder_init__assert_default_path(pill):
    assert pill.path() == Path.home()/'die'

def test_kill_clear__boolean(pill):
    pill.clear()
    assert not pill
    pill.kill()
    assert pill
    pill.clear()
    assert not pill

def test_kill_clear__path_existance(pill):
    pill.clear()
    assert not pill.path().exists()
    pill.kill()
    assert pill.path().exists()
    pill.clear()
    assert not pill.path().exists()





#!/usr/bin/env python3
from pathlib import Path

class KillPill:
    def __init__(self, path=None):
        if path is None:
            self._path = Path.home()/'die'
        else:
            self._path = Path(path)
    def path(self):
        return self._path
    def __bool__(self):
        return self._path.exists()
    def kill(self):
        self._path.touch()
    def clear(self):
        if self._path.exists():
            self._path.unlink()



#!/usr/bin/env python3
import time

class Timer:
    def __repr__(self):
        d = int(self._duration)
        p = int(self.percent()*100)
        c = self.count()
        return f'<Timer({d}):{p}%:count={c}>'

    def poll(self):
        c = self.count()
        if not c in self._counts:
            self._counts.append(c)
            return True
        return False

    def __init__(self, duration, count, tick):
        self._duration = duration
        self._count = count
        self._tick = tick
        self._counts = []
        self._t0 = time.time()
        self._deadline = self._t0 + duration

    def remaining(self):
        return self._deadline - time.time()

    def tick(self):
        time.sleep(self._tick)

    def percent(self):
        return self.remaining() / self._duration

    def count(self):
        return int(self.percent() * self._count )

    def __bool__(self):
        return self.remaining() > 0




#!/usr/bin/env python3
from pathlib import Path
import tempfile
import time
import sys

class FlagFolder:
    def __init__(self, path):
        self._path = Path(path)
        if not self._path.exists():
            self._path.mkdir(parents=True, exist_ok=True)

    def flags(self):
        return sorted([x.name for x in self._path.glob('*')])

    def stat(self,flag):
        path = self._path/flag
        return path.exists()

    def set(self,flag):
        path = self._path/flag
        path.exists() or path.touch()

    def clear(self,flag):
        path = self._path/flag
        path.exists() and path.unlink()

    def clear_all(self):
        for path in self._path.glob('*'):
            path.unlink()




#!/usr/bin/env python3
import time

import pytest

from bh_util.timer import Timer

def test_dunder_short():
    duration=0.05
    tick=0.0001
    count=5
    timer = Timer(duration=duration,count=count, tick=tick)
    acc = []
    t0 = time.time()
    while timer:
        if timer.poll():
            acc.append(timer.count())
        timer.tick()
    assert time.time()-t0 == pytest.approx(duration, rel=1e-2)
    assert acc==[4,3,2,1,0]

def test_dunder_long():
    duration=0.5
    tick=0.0001
    count=5
    timer = Timer(duration=duration,count=count, tick=tick)
    acc = []
    t0 = time.time()
    while timer:
        if timer.poll():
            acc.append(timer.count())
        timer.tick()
    assert time.time()-t0 == pytest.approx(duration, rel=1e-2)
    assert acc==[4,3,2,1,0]



#!/usr/bin/env python3
import sys

class Debug:
    def __init__(self, stat=False):
        if stat: self.turn_on()
        else: self.turn_off()
    def turn_on(self):  sys._bh_debug = True
    def turn_off(self): sys._bh_debug = False
    def __bool__(self): return sys._bh_debug
    def err(self, text, end='\n'):
        if self:
            sys.stderr.write(text + end)
            sys.stderr.flush()

